function [Sigma,ier] = Sigma_Compute(b,c,d,q,hfrac_vec)
% Compute Sigma for "b,c,d" model for q cosine transforms and forecast
% horizon hfrac. Note hfrac_vec is a vector that contains the forecast
% horizons in fractions of the sample size
%
 
 % Approximation are based on T "sample" observations 
 %    Use Small T for general model because it requires evaluation of
 %    hypergeometric which is slow and relatively innacurate for large T
 %    (Note ... try to modify this finding alternative approxiamation).
 %    Large T is fine for special cases.
 %    ier = 0 (no problems); ier = 1 (negative eigenvalue, corrected)
 T_general = 150;
 T_special = 600;
 
 % -------------  Step 1; Compute Covariance Matrix for I(d) Component 
    % A. Compute I(d) Component ... look at some special Cases
     if c < .00001;  % Use fractional model
        T = T_special;
        h_T_vec = floor(hfrac_vec*T);
        h_T_max = max(h_T_vec);
        TA = T + h_T_max;       % T augmented with forecast period
        cov_Id = frac_cov(TA,d);
     else;
        if d == 1;    % AR(1) Model with Zero Initial Condition
             T = T_special;
             rho = 1 - c/T;
             rho = max([0;rho]);
             h_T_vec = floor(hfrac_vec*T);
             h_T_max = max(h_T_vec);
             TA = T + h_T_max;       % T augmented with forecast period
             cov_Id = AR1_cov(TA,rho);
        else;
             T = T_general;
             rho = 1 - c/T;
             rho = max([0;rho]);
             h_T_vec = floor(hfrac_vec*T);
             h_T_max = max(h_T_vec);
             TA = T + h_T_max;       % T augmented with forecast period
             cov_Id = frac_dif_cov(TA,rho,d);
        end;
     end;
    % -- B. Add I(0) Components  
     scale_I0 = (b*(T^d))^2;         % Scale for I(0) component
     covmat = cov_Id + scale_I0*eye(TA);   
     
 % ------------------ Step 2: Compute PSI Matrix and Extension for xbar(t+1:t+h)
   psi = psi_compute(T,q); % Cosine Transformations
   % Matrix to subtract in-sample mean 
   a = zeros(1,TA);
   a(1,1:T) = ones(1,T)/T;  
   tmp = repmat(a,TA,1);
   const_adj = eye(TA)-tmp;  
   % Matrix for holding weights 
   dp_dm = zeros(q+size(hfrac_vec,1),TA);
   dp_dm(1:q,1:T) = psi';
   for i = 1:size(hfrac_vec,1);
       h_T = h_T_vec(i);
       dp_dm(q+i,T+1:T+h_T) = ones(h_T,1)/h_T;
   end;
   dp_dm = dp_dm*const_adj;  % These are the weights
  
 % ---------------- Step 3: Compute Sigma
   Sigma = dp_dm*covmat*dp_dm';
   Sigma = (T^(1-2*d))*Sigma;
   
 % ---------------- Step 4: Verify that Sigma is PD
   ier = 0;
   tmp = eig(Sigma);
   if min(tmp) < 0;
       ier = 1;
       %fprintf('... Estimated Sigma has negative eigenvalue \n')
       %fprintf('b,c,d = %6.4f,%6.2f,%6.2f \n',b,c,d)
       %fprintf('     ....... Correcting Matrix .....\n ');
       Sigma_2 = Sigma*Sigma';
       [V,D2]=eig(Sigma_2);
       D_vec = sqrt(diag(D2));
       D_mat = diag(D_vec);
       Sigma = V*D_mat*V';
   end;
 
  % ------------- Step 5: Choose Scale so that Sigma(1,1) = 1;
  	Sigma = Sigma/(Sigma(1,1));     
   
end

