function [covmat] = frac_dif_cov(T,rho,d)
% Compute Covariance Matrix for fractional process with AR parameter rho
% Note ... This computes stationary covariance matrix for -0.5 < d < 0.5 and then "rho-accumulates" for d > 0.5 -- same strategy as fractional model
%
% Only for stable valus of rho
small = 1.0e-10;
if abs(rho) > 1-small;
    error('rho >= 1-small number')'
end;
% d must be between - 0.5 and 1.5 
if d < -0.5;
 error('d cannot be less than - 0.5');
end;
if d > 1.5;
 error('d cannot be greater than 1.5')
end;

% Use I(0) model if d is very close to zero
if abs(d) < 0.000001;
    covmat = eye(T);
else;
  %  Compute autocovariances
  acv = NaN*zeros(T,1);  % Vector for storing ACV 0-(T-1)
  acv_ind = (0:T-1)';    % Index values of ACVs
  const_rho = rho.^acv_ind;
  t1 = min([100;T]);
   const_gamma = NaN*zeros(T,1); 
   const_gamma(1:t1) = gamma(d+acv_ind(1:t1))./gamma(1+acv_ind(1:t1));
  if T > t1;
   anum = app_log_gamma(d+acv_ind(t1+1:T));
   aden = app_log_gamma(1+acv_ind(t1+1:T));
   const_gamma(t1+1:T) = exp(anum-aden);
   %const_gamma(t1+1:T) = acv_ind(t1+1:T).^(d-1);  % Sterlings approximation
  end;
  const_gamma = const_gamma/gamma(d);
  const = const_rho.*const_gamma;
   % Parameters for hypergeometric distribution
   a = d + acv_ind;
   b = d;
   c = 1 + acv_ind;
   z = rho^2;
   for i = 1:T;
     acv(i) = hypergeom([a(i),b],c(i),z);
   end;
   acv = const.*acv;
   covmat = acv2cov(acv);
end;

end

